package com.example.android_sdk_tutorial;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Set;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import com.idtechproducts.device.*;

public class MainActivity extends Activity implements  OnReceiverListener{

	// declaring the instance of the UniPayReader;
	private IDT_KioskIII myUniPayReader = null;
	private TextView connectStatusTextView;  
	private TextView textLog;
	private TextView lcdLog;
	private Button btnGetFirmware;
	private Button btnStartEMV;
	private Button btnCancelEMV;
	private Handler handler = new Handler();
    private boolean isReaderConnected = false;
    private String info = "";
	private String detail = "";
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		handler = new Handler();		
		btnStartEMV = (Button)findViewById(R.id.btn_StartEMV);
		btnCancelEMV = (Button)findViewById(R.id.btn_CancelEMV);
		btnGetFirmware = (Button)findViewById(R.id.btn_getFirmware);
	    textLog = (TextView)findViewById(R.id.textLog);
	    lcdLog = (TextView)findViewById(R.id.lcdLog);
	    connectStatusTextView = (TextView)findViewById(R.id.status_text);
		if(myUniPayReader!=null){
			myUniPayReader.unregisterListen();
			myUniPayReader.release();
			myUniPayReader = null;
		}
		myUniPayReader = new IDT_KioskIII(this,this);
		myUniPayReader.registerListen();
		loadXMLfile();
		

		btnGetFirmware.setOnClickListener(new Button.OnClickListener(){  
	        public void onClick(View v) {
	        	info = "Getting Firmware\n";
	        	detail = "";
	        	handler.post(doUpdateStatus);
	        	StringBuilder sb = new StringBuilder();
	            int ret = myUniPayReader.device_getFirmwareVersion(sb);
	            if (ret == ErrorCode.SUCCESS) {
					info += "Firmware Version: " + sb.toString();
					detail = "";
					handler.post(doUpdateStatus);
				}
				else {
					info += "GetFirmwareVersion: Failed\n";
					info += "Status: "+ myUniPayReader.device_getResponseCodeString(ret)+"";
					detail = "";
					handler.post(doUpdateStatus);							
					}
	        }  
	    });
		
		btnStartEMV.setOnClickListener(new Button.OnClickListener(){  
	        public void onClick(View v) {;
	        detail = "";
	        info = "Starting CTLS Transaction\n";
        	handler.post(doUpdateStatus);
				myUniPayReader.ctls_startTransaction(1.00, 0.00, 0, 30, null);
	        }  
	    });
		
		btnCancelEMV.setOnClickListener(new Button.OnClickListener(){  
	        public void onClick(View v) {
	        	detail = "";
		        info = "Canceling CTLS Transaction\n";
	        	handler.post(doUpdateStatus);
				myUniPayReader.ctls_cancelTransaction();
	        }  
	    });
		
		
	}

	@Override
	public void ICCNotifyInfo(byte[] arg0, String arg1) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void LoadXMLConfigFailureInfo(int arg0, String arg1) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void autoConfigCompleted(StructConfigParameters arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void autoConfigProgress(int arg0) {
		// TODO Auto-generated method stub
		
	}
	
	private Runnable doUpdateLabel = new Runnable()
	{
	    public void run()
	    {
	        if(!isReaderConnected){
	            connectStatusTextView.setText("KIOSKIII DISCONNECTED");
	        }
	        else{
	            connectStatusTextView.setText("KIOSKIII CONNECTED");
	        }
	    }
	};
	@Override
	public void deviceConnected() {
		isReaderConnected = true;
		handler.post(doUpdateLabel);		
	}

	@Override
	public void deviceDisconnected() {
		isReaderConnected = false;
	    handler.post(doUpdateLabel);		
	}
	
	
	@Override
	public void emvTransactionData(IDTEMVData emvData) {
		if (emvData == null) return;
		detail += Common.emvErrorCodes(emvData.result);
		detail += "\r\n";
		
		detail += "\r\nCTLS Transaction response:\r\n";
		if (emvData.unencryptedTags != null && !emvData.unencryptedTags.isEmpty())
		{
			detail += "Unencrypted Tags:\r\n";
			Set<String> keys = emvData.unencryptedTags.keySet();
			for(String key: keys){
				detail += key + ": ";
				byte[] data = emvData.unencryptedTags.get(key);
				detail += Common.getHexStringFromBytes(data) + "\r\n";
			}
		}
		if (emvData.maskedTags!= null && !emvData.maskedTags.isEmpty())
		{
			detail += "Masked Tags:\r\n";
			Set<String> keys = emvData.maskedTags.keySet();
			for(String key: keys){
				detail += key + ": ";
				byte[] data = emvData.maskedTags.get(key);
				detail += Common.getHexStringFromBytes(data) + "\r\n";
			}
		}
		if (emvData.encryptedTags != null && !emvData.encryptedTags.isEmpty())
		{
			detail += "Encrypted Tags:\r\n";
			Set<String> keys = emvData.encryptedTags.keySet();
			for(String key: keys){
				detail += key + ": ";
				byte[] data = emvData.encryptedTags.get(key);
				detail += Common.getHexStringFromBytes(data) + "\r\n";
			}
		}
		handler.post(doUpdateStatus);

	
	}




	@Override
	public void msgAudioVolumeAjustFailed() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void msgRKICompleted(String arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void msgToConnectDevice() {
		// TODO Auto-generated method stub
		
	}

	private Runnable doUpdateStatus = new Runnable()
	{
		public void run()
		{		
			lcdLog.setText(info);
			textLog.setText(detail);
		}
	};
	@Override
	public void swipeMSRData(IDTMSRData card) {
		
	}

	@Override
	public void timeout(int arg0) {
		// TODO Auto-generated method stub
		
	}
	
	private String getXMLFileFromRaw(String fileName ,int res){
		//the target filename in the application path
		String fileNameWithPath = null;
		fileNameWithPath = fileName;
		String newFilename = fileName;
	
		try {
			InputStream in = getResources().openRawResource(res);
			int length = in.available();
			byte [] buffer = new byte[length];
			in.read(buffer);    	   
			in.close();
			deleteFile(fileNameWithPath);
			FileOutputStream fout = openFileOutput(fileNameWithPath, MODE_PRIVATE);
			fout.write(buffer);
			fout.close();
    	   
			// to refer to the application path
			File fileDir = this.getFilesDir();
			fileNameWithPath = fileDir.getParent() + java.io.File.separator + fileDir.getName();
			fileNameWithPath += java.io.File.separator+newFilename;
	   	   
		} catch(Exception e){
			e.printStackTrace();
			fileNameWithPath = null;
		}
		return fileNameWithPath;
	}
	
	private String getConfigurationFileFromRaw( ){
		return getXMLFileFromRaw("idt_unimagcfg_default.xml",R.raw.idt_unimagcfg_default);
		}
	private boolean isFileExist(String path) {
    	if(path==null)
    		return false;
	    File file = new File(path);
	    if (!file.exists()) {
	      return false ;
	    }
	    return true;
    }
	private void loadXMLfile(){
		
	    //load the XML configuratin file
	    String fileNameWithPath = getConfigurationFileFromRaw();
	    if(!isFileExist(fileNameWithPath)) { 
	    	fileNameWithPath = null; 
	    }        	
	    /////////////////////////////////////////////////////////////////////////////////
		// Network operation is prohibited in the UI Thread if target API is 11 or above.
		// If target API is 11 or above, please use AsyncTask to avoid errors.
	    myUniPayReader.config_setXMLFileNameWithPath(fileNameWithPath);
	    Log.d("Demo Info >>>>>","loadingConfigurationXMLFile begin.");
	    myUniPayReader.config_loadingConfigurationXMLFile(true);
	    /////////////////////////////////////////////////////////////////////////////////
	}

	@Override
	public void dataInOutMonitor(byte[] arg0, boolean arg1) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void msgBatteryLow() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void lcdDisplay(int mode, String[] lines, int timeout) {
		// TODO Auto-generated method stub
		
	}
	
	
	

	
	
}
